$compile unit

$error all-
$lib all-

defint a-z


'-----------------------------------------------------------------------
'
' splitfn           splits a complete file name into drive, directory,
'                   filename and extension components.  any missing
'                   components are returned as null strings.  file name
'                   is truncated to 8 characters and extension is truncated
'                   to 3 characters (plus dot).  a path of ".." or "."
'                   will be returned as a directory.  NOTE:  this is a
'                   text-only routine.  no checking is performed on the
'                   path for valid drive names, directories, etc.
'
' input parameters:
'
'    path$          path/file string to be split
'
' output parameters: (set to "" if not specified in path$)
'
'    drive$         drive code, includes ":"
'    DIRECTORY$           directory path, includes trailing "\"
'    file$          file name
'    ext$           extension string, includes "."
'
sub splitfn(path$,drive$,DIRECTORY$,file$,ext$)  PUBLIC
      local temp$
      local i%                          ' start of directory
      local j%                          ' start of file name
      local k%                          ' start of extension
      local l%                          ' end of path

      temp$ = ucase$(path$)             ' get local copy of path
      while left$(temp$,1)=" ": temp$ = right$(temp$,len(temp$)-1): wend
      while right$(temp$,1)=" ": temp$ = left$(temp$,len(temp$)-1): wend
      drive$ = ""
      DIRECTORY$ = ""
      file$ = ""
      ext$ = ""

      if temp$="." or temp$=".." then   ' special case
         DIRECTORY$ = temp$
         exit sub
      end if

      l% = len(temp$)+1                 ' set end of path
      i% = 2                            ' find colon
      while i%<=l and mid$(temp$,i%-1,1)<>":": i% = i%+1: wend
      if mid$(temp$,i%-1,1)<>":" then i% = 1
      j% = l%                           ' find rightmost backslash
      while j%>i% and mid$(temp$,j%-1,1)<>"\": j% = j%-1: wend
      k% = l%                           ' find extension dot
      while k%>j% and mid$(temp$,k%,1)<>".": k% = k%-1: wend
      if mid$(temp$,k%,1)<>"." then k% = l%  ' no extension dot found

      drive$ = mid$(temp$,1,i%-1)       ' extract drive
      DIRECTORY$ = mid$(temp$,i%,j%-i%)       ' extract directory
      file$ = left$(mid$(temp$,j%,k%-j%),8)  ' extract file name
      ext$ = left$(mid$(temp$,k%,l%-k%),4)   ' extract extension
end sub