' UTILITY.LIB  Other Utility Routines 

'Ŀ
' SUBROUTINE Parse(text$,word$)                                              
'    Make initial CALL with sentence/COMMAND$/etc. in TEXT$; returns first   
'    WORD$.  Additional CALLs made with a null TEXT$; returns next WORD$.    
'

   SUB Parse(text$,word$)
      STATIC temp$,i%,max%
      LOCAL  start%
      IF LEN(text$) THEN
         temp$=text$+" "
         i%=1
         max%=LEN(temp$)
      END IF
      word$=""
      WHILE MID$(temp$,i%,1)=" " AND i%<max%
         INCR i%
      WEND
      start%=i%
      IF i%=max% THEN EXIT SUB
      WHILE MID$(temp$,i%,1)<>" " AND i%<=max%
         INCR i%
      WEND
      word$=MID$(temp$,start%,i%-start%)
   END SUB


'Ŀ
' FUNCTION Soundex$(word$)                                                   
'    Creates "sound-index" string of digits from word/last name/etc.         
'    Can be used for sound-alike name matching, typo/spell check, etc.       
'    Max. length of returned string is the number of consonants in WORD$.    
'

   DEF FnSoundex$(word$)
      LOCAL i%,lt%,sndx$,c$,lastc$,temp$
      lt%=LEN(word$)
      FnSoundex$="" : IF lt%=0 THEN EXIT DEF
      lastc$=""
      sndx$=""
      temp$=UCASE$(word$)
      FOR i%=1 TO lt%
         c$=MID$(temp$,i%,1)
         IF INSTR(lastc$,c$) THEN
            lastc$=lastc$
         ELSEIF INSTR("AEHIOUWY",c$) THEN
            lastc$="AEHIOUWY"
         ELSEIF INSTR("BFPV",c$) THEN
            sndx$=sndx$+"1" : lastc$="BFPV"
         ELSEIF INSTR("CGJKQSXZ",c$) THEN
            sndx$=sndx$+"2" : lastc$="CGJKQSXZ"
         ELSEIF INSTR("DT",c$) THEN
            sndx$=sndx$+"3" : lastc$="DT"
         ELSEIF c$="L" THEN
            sndx$=sndx$+"4" : lastc$="L"
         ELSEIF INSTR("MN",c$) THEN
            sndx$=sndx$+"5" : lastc$="MN"
         ELSEIF c$="R" THEN
            sndx$=sndx$+"6" : lastc$="R"
         END IF
      NEXT
      FnSoundex$=sndx$
   END DEF


'Ŀ
' FUNCTION Extract$(lookup$,delim$,index%)                                   
'    Simple table lookup function.  Returns the INDEX% numbered item from    
'    the list LOOKUP$, whose items are delimited by DELIM$.  If there are    
'    not at least INDEX% items in the list a null string is returned.        
'

   DEF FnExtract$(lookup$,delim$,index%)
      LOCAL i%,j%,count%,lt%,temp$
      FnExtract$=""
      IF index%<1 THEN EXIT DEF
      IF lookup$="" THEN EXIT DEF
      IF LEN(delim$)<>1 THEN EXIT DEF
      j%=0
      count%=0
      temp$=lookup$+delim$
      lt%=LEN(temp$)
      DO
         i%=j%+1
         j%=INSTR(i%,temp$,delim$)
         INCR count%
      LOOP UNTIL count%=index% OR j%=lt%
      IF count%=index% THEN FnExtract$=MID$(lookup$,i%,j%-i%)
   END DEF


'Ŀ
' SUBROUTINE PrintScreen                                                     
'    Call DOS screen print interrupt.                                        
'

   SUB PrintScreen
      CALL INTERRUPT &h05
   END SUB


'Ŀ
' FUNCTION DirTimeFormat$(hours%,minutes%)                                   
'    Formats HOURS% (0-24) and MINUTES% in 12-hour am-pm directory form      
'    HH:MMx, with a leading blank instead of "0" and x either "a" or "p".    
'

   DEF FnDirTimeFormat$(hours%,minutes%)
      LOCAL hr%,ap$
      ap$="a" : IF hours%>11 THEN ap$="p"
      hr%=(hours% MOD 12) : IF hr%=0 THEN hr%=12
      FnDirTimeFormat$=RIGHT$(STR$(hr%),2)+":"+MID$(STR$(100+minutes%),3)+ap$
   END DEF


'Ŀ
' FUNCTION DirDateFormat$(year%,month%,day%)                                 
'    Formats YEAR%, MONTH% and DAY% in directory form MM-DD-YY, with a       
'    leading blank instead of "0".                                           
'

   DEF FnDirDateFormat$(year%,month%,day%)
      LOCAL temp$
      temp$=RIGHT$(STR$(month%),2)+"-"+MID$(STR$(100+day%),3)
      FnDirDateFormat$=temp$+STR$(-(year%-1900))
   END DEF


'Ŀ
' SUBROUTINE MDY.to.Julian(month%,day%,year%,julian&,weekday%)               
'    Return JULIAN& date value for MONTH%-DAY%-YEAR%.  Also returns the      
'    WEEKDAY% (1=Sunday to 7=Saturday).  Allows date arithmetic.             
'

   SUB MDY.to.Julian(month%,day%,year%,julian&,weekday%)
      LOCAL temp1%,temp2%,temp3%,temp4&
      julian&=FIX(365.2422#*year%+30.44*(month%-1)+day%+1)
      temp1%=month%-2-12*(month%<3)
      temp2%=year%+(month%<3)
      temp3%=FIX(temp2%/100)
      temp2%=temp2%-100*temp3%
      weekday%=FIX(2.61*temp1%-.2)+day%+temp2%+FIX(temp2%/4)
      weekday%=(weekday%+INT(temp3%/4)-temp3%-temp3%+77) MOD 7 + 1
      temp4&=julian&-7*FIX(julian&/7)
      julian&=julian&-temp4&+weekday%+7*(temp4&<weekday%-1)+1721060#
   END SUB


'Ŀ
' SUBROUTINE Julian.to.MDY(julian&,month%,day%,year%,weekday%)               
'    Returns MONTH%, DAY%, YEAR%, and WEEKDAY% (1=Sunday to 7=Saturday)      
'    for speicified date value JULIAN&.  Allows date arithmetic.             
'                                                                            
'    ***  This routine calls MDY.to.Julian - use both if using this  ***     
'

   SUB Julian.to.MDY(julian&,month%,day%,year%,weekday%)
      LOCAL jtemp&,yr.temp%,wd.temp%
      yr.temp%=FIX((julian&-1721061#)/365.25+1)
      INCR yr.temp%
      DO
         DECR yr.temp%
         CALL MDY.to.Julian(1%,1%,yr.temp%,jtemp&,wd.temp%)
      LOOP WHILE jtemp&>julian&
      mo.temp%=FIX((julian&-jtemp&)/29+1)
      INCR mo.temp%
      DO
         DECR mo.temp%
         CALL MDY.to.Julian(mo.temp%,1%,yr.temp%,jtemp&,wd.temp%)
      LOOP WHILE jtemp&>julian&
      da.temp%=CINT(julian&-jtemp&+1)
      CALL MDY.to.Julian(mo.temp%,da.temp%,yr.temp%,jtemp&,wd.temp%)
      month%=mo.temp%
      day%=da.temp%
      year%=yr.temp%
      weekday%=wd.temp%
   END SUB

'Ŀ
' SUBROUTINE CHECKPRINTER(ROW%,COLUMN%,FOREGROUND COLOR,BACKGROUND COLOR)    
'    CHECKS PRINTER AND PRINTS ERROR MESSAGE AT ROW% , COLUMN%               
'    MUST HAVE PW200 WINDOWING PROGRAM                                       
'

SUB CHECKPRINTER(PROW%,PCOL%,PCOLF%,PCOLB%)
	LOCAL PRTABORT,X,ERR1$

CKPRTER:
	PRTABORT=1
	FOR X=1 TO 1500
	PRTOK=PRTEST(1)
	IF PRTOK<>0 THEN PRTABORT=0:EXIT FOR
	NEXT X
	IF PRTABORT=0 THEN EXIT SUB
	BEEP
	CALL OPENWIN(PROW%,PCOL%,5,30,PCOLF%,PCOLB%,2,0,0,0)
	CALL PRINTCWIN(1," PLEASE CHECK PRINTER ")
	CALL PRINTCWIN(3," TYPE  ( C )  TO CONTINUE ")
CKPRTANSW:
	DO
		ERR1$=INKEY$
	LOOP WHILE ERR1$=""
	IF ERR1$="C" OR ERR1$="c" THEN
		CALL CLOSEWIN
		GOTO CKPRTER
	ELSE
		GOTO CKPRTANSW
	END IF
END SUB


'(EOF: UTILITY.LIB)
